@@ -1,7 +1,7 @@
 ;;; -*- coding: iso-2022-7bit; byte-compile-dynamic: t -*-
 ;;; mucs.el --- Mule-UCS setup file.
 
-;; Copyright (C) 1997-2000 Miyashita Hisashi
+;; Copyright (C) 1997-2001 Miyashita Hisashi
 
 ;; Keywords: mule, multilingual, 
 ;;           character set, coding-system, ISO10646, Unicode
@@ -39,7 +39,7 @@
 ;; 1.80->IMADEGAWA
 ;; 1.90->DAIRI
 ;; 2.00->Kyoto Univ.
-(defconst mucs-version "0.83 (JISHOJI:$B;|>H;{(B)")
+(defconst mucs-version "0.84 (KOUGETSUDAI:$B8~7nBf(B)")
 
 ;; For error handling.
 
@@ -435,7 +435,7 @@ Therefore, you can use these without any Mule-UCS modules.
        'mucs-compiled-code code)))
 
 ;;;
-;;;
+;;; Coding system API
 ;;;
 
 (defmacro mucs-define-coding-system
@@ -475,4 +475,114 @@ Therefore, you can use these without any Mule-UCS modules.
 	(t
 	 (error "This Emacs has no Mule feature."))))
 
+;;;
+;;; Encoding/Decoding API.
+;;;
+
+;;; Symbol's property
+;; mucs-encoding-backend
+;; mucs-encoding-default-backend
+;; mucs-decoding-backend
+;; mucs-decoding-default-backend
+
+;; Currently, supported restriction classes are:
+;;   charset
+;; only.
+
+(defsubst mucs-get-representation-encoding-backend
+  (representation restriction)
+  (if restriction
+      (or (and (listp restriction)
+	       (cdr
+		(assq (car restriction)
+		      (get representation 'mucs-encoding-backend))))
+	  (error "Invalid restriction:%S" restriction))
+    (get representation 'mucs-encoding-default-backend)))
+
+(defsubst mucs-get-representation-decoding-backend
+  (representation restriction)
+  (if restriction
+      (or (and (listp restriction)
+	       (cdr
+		(assq (car restriction)
+		      (get representation 'mucs-decoding-backend))))
+	  (error "Invalid restriction:%S" restriction))
+    (get representation 'mucs-decoding-default-backend)))
+
+(defun mucs-register-representation-encoding-backend
+  (representation restriction-category backend)
+  (let (alist slot)
+    (cond ((eq restriction-category 'nil)
+	   (put representation 'mucs-encoding-default-backend
+		(list backend)))
+	  ((symbolp restriction-category)
+	   (setq alist (get representation 'mucs-encoding-backend)
+		 slot (assq restriction-category representation))
+	   (if slot
+	       (put representation 'mucs-encoding-backend
+		    (cons (cons restriction-category backend)
+			  alist))
+	     (setcdr slot backend)))
+	  (t
+	   (error "Invalid restriction category:%S." restriction-category)))))
+
+(defun mucs-register-representation-decoding-backend
+  (representation restriction-category backend)
+  (let (alist slot)
+    (cond ((eq restriction-category 'nil)
+	   (put representation 'mucs-decoding-default-backend
+		(list backend)))
+	  ((symbolp restriction-category)
+	   (setq alist (get representation 'mucs-decoding-backend)
+		 slot (assq restriction-category representation))
+	   (if slot
+	       (put representation 'mucs-decoding-backend
+		    (cons (cons restriction-category backend)
+			  alist))
+	     (setcdr slot backend)))
+	  (t
+	   (error "Invalid restriction category:%S." restriction-category)))))
+
+(defun encode-char (char representation &optional restriction)
+  "Return character representation(code-point, explanation, category, attribute
+and so on.) in REPRESENTATION that corresponds to CHAR.
+Return nil if CHAR cannot be represented.
+Available representation list can be obtained by mucs-representation-list.
+
+Optional argument RESTRICTION specifies a way to map CHAR to
+representation.  Its interpretation depends on the given
+REPRESENTATION.  If not specified, the default restriction of
+REPRESENTATION is used."
+  (let ((fs (mucs-get-representation-encoding-backend
+	     representation restriction))
+	ret)
+    (while
+	(and fs
+	     (not (setq ret
+			(funcall
+			 (car fs)
+			 char representation restriction))))
+      (setq fs (cdr fs)))
+    ret))
+
+(defun decode-char (representation object &optional restriction)
+  "Return a character represented by OBJECT in view of REPRESENTATION.
+Return nil if OBJECT cannot be mapped to only one character.
+Available representation list can be obtained by mucs-representation-list.
+Optional argument RESTRICTION specifies a way to map OBJECT to
+a character.  Its interpretation depends on the given
+REPRESENTATION.  If not specified, the default restriction of REPRESENTATION
+is used."
+  (let ((fs (mucs-get-representation-decoding-backend
+	     representation restriction))
+	ret)
+    (while
+	(and fs
+	     (not (setq ret
+			(funcall
+			 (car fs)
+			 representation object restriction))))
+      (setq fs (cdr fs)))
+    ret))
+
 (provide 'mucs)
