@@ -27,6 +27,17 @@
 
 (require 'unicode)
 (require 'tae)
+
+;; Because of the incompatibility between XEmacs and Emacs on
+;; number literal, we use cn macro that translates a string in
+;; C-notation to number.  But if all hexadeciman numbers are
+;; described by cn macro, the code seems very dirty, so I use
+;; cn macro only on numbers more than 0xFF.  Strictly, it is
+;; not reasonable, because we cannot apply arithmetic
+;; transformation to characters. But we can use ?\x00-?\xFF, because
+;; all of them are mapped into 0-255 in CCL environments of
+;; both XEmacs and Emacs.
+
 (require 'trans-util)
 
 ;;; Buffer magnification scales for UTFs.
@@ -55,55 +66,55 @@
 (defvar utf-8-ccl-encode
   `((if (r0 < ?\x80)
 	((write r0))
-      (if (r0 < ?\x800)
-	  ((write ((r0 >> 6) | ?\xc0))
-	   (write ((r0 & ?\x3f) | ?\x80)))
-	(if (r0 < ?\x10000)
-	    ((write ((r0 >> 12) | ?\xe0))
-	     (write (((r0 >> 6) & ?\x3f) | ?\x80))
-	     (write ((r0 & ?\x3f) | ?\x80)))
-	  (if (r0 < ?\x200000)
-	      ((write ((r0 >> 18) | ?\xf0))
-	       (write (((r0 >> 12) & ?\3f) | ?\x80))
-	       (write (((r0 >> 6) & ?\x3f) | ?\x80))
-	       (write ((r0 & ?\x3f) | ?\x80)))
-	    (if (r0 < ?\x4000000)
-		((write ((r0 >> 24) | ?\xf8))
-		 (write (((r0 >> 18) & ?\x3f) | ?\x80))
-		 (write (((r0 >> 12) & ?\x3f) | ?\x80))
-		 (write (((r0 >> 6) & ?\x3f) | ?\x80))
+      (if (r0 < ,(cn "0x800"))
+	  ((write ((r0 >> 6) | ?\xC0))
+	   (write ((r0 & ?\x3F) | ?\x80)))
+	(if (r0 < ,(cn "0x10000"))
+	    ((write ((r0 >> 12) | ?\xE0))
+	     (write (((r0 >> 6) & ?\x3F) | ?\x80))
+	     (write ((r0 & ?\x3F) | ?\x80)))
+	  (if (r0 < ,(cn "0x200000"))
+	      ((write ((r0 >> 18) | ?\xF0))
+	       (write (((r0 >> 12) & ?\3F) | ?\x80))
+	       (write (((r0 >> 6) & ?\x3F) | ?\x80))
+	       (write ((r0 & ?\x3F) | ?\x80)))
+	    (if (r0 < ,(cn "0x4000000"))
+		((write ((r0 >> 24) | ?\xF8))
+		 (write (((r0 >> 18) & ?\x3F) | ?\x80))
+		 (write (((r0 >> 12) & ?\x3F) | ?\x80))
+		 (write (((r0 >> 6) & ?\x3F) | ?\x80))
 		 (write ((r0 & ?\x3f) | ?\x80)))
-	      ((write ((r0 >> 30) | ?\xfc))
-	       (write (((r0 >> 24) & ?\x3f) | ?\x80))
-	       (write (((r0 >> 18) & ?\x3f) | ?\x80))
-	       (write (((r0 >> 12) & ?\x3f) | ?\x80))
-	       (write (((r0 >> 6) & ?\x3f) | ?\x80))
+	      ((write ((r0 >> 30) | ?\xFC))
+	       (write (((r0 >> 24) & ?\x3F) | ?\x80))
+	       (write (((r0 >> 18) & ?\x3F) | ?\x80))
+	       (write (((r0 >> 12) & ?\x3F) | ?\x80))
+	       (write (((r0 >> 6) & ?\x3F) | ?\x80))
 	       (write ((r0 & ?\x3f) | ?\x80))))))))))
 
 (defvar utf-8-ccl-decode
   `((read-if (r0 >= ?\x80)
-	((if (r0 < ?\xe0)
+	((if (r0 < ?\xE0)
 	     ((read r4)
-	      (r4 &= ?\x3f)
-	      (r0 = (((r0 & ?\x1f) << 6) | r4)))
-	   (if (r0 < ?\xf0)
+	      (r4 &= ?\x3F)
+	      (r0 = (((r0 & ?\x1F) << 6) | r4)))
+	   (if (r0 < ?\xF0)
 	       ((read r4 r6)
-		(r4 = ((r4  & ?\x3f) << 6))
-		(r6 &= ?\x3f)
-		(r0 = ((((r0 & ?\xf) << 12) | r4) | r6)))
-	     (if (r0 < ?\xf8)
+		(r4 = ((r4  & ?\x3F) << 6))
+		(r6 &= ?\x3F)
+		(r0 = ((((r0 & ?\x0F) << 12) | r4) | r6)))
+	     (if (r0 < ?\xF8)
 		 ((read r1 r4 r6)
-		  (r1 = ((r1  & ?\x3f) << 12))
-		  (r4 = ((r4  & ?\x3f) << 6))
-		  (r6 &= ?\x3f)
-		  (r0 = (((((r0 & ?\x7) << 18) | r1) | r4) | r6)))
-	       (if (r0 < ?\xfc)
+		  (r1 = ((r1  & ?\x3F) << 12))
+		  (r4 = ((r4  & ?\x3F) << 6))
+		  (r6 &= ?\x3F)
+		  (r0 = (((((r0 & ?\x07) << 18) | r1) | r4) | r6)))
+	       (if (r0 < ?\xFC)
 ;;;; MUCS can't read any numbers lager than 24bit
 		   ((read r0 r1 r4 r6)
-		    (r1 = ((r1  & ?\x3f) << 12))
-		    (r4 = ((r4  & ?\x3f) << 6))
-		    (r6 &= ?\x3f)
-		    (r0 = (((((r0 & ?\x3f) << 18) | r1) | r4) | r6)))
+		    (r1 = ((r1  & ?\x3F) << 12))
+		    (r4 = ((r4  & ?\x3F) << 6))
+		    (r6 &= ?\x3F)
+		    (r0 = (((((r0 & ?\x3F) << 18) | r1) | r4) | r6)))
 		 (r0 = 0)))))))))
 
 (mucs-type-register-serialization
@@ -133,7 +144,7 @@
 ;UTF 16 -----------------------------------------------
 
 (defun utf-16-ccl-surrogate-pair-p (reg)
-  `((,reg & ?\xf800) == ?\xd800))
+  `((,reg & ,(cn "0xF800")) == ,(cn "0xD800")))
 
 (mucs-ccl-internal-state-reserve 'utf-16-little-endian-p 1)
 
@@ -164,9 +175,9 @@
 		((if ,(mucs-ccl-check-internal-state 'utf-16-little-endian-p)
 		     ((read r6 r4))
 		   ((read r4 r6)))
-		 (r0 = (((r0 & ?\x3ff) + ?\x40) << 10))
-		 (r6 &= ?\x3f)
-		 (r4 = ((r4 & ?\x3) << 6) | r6)
+		 (r0 = (((r0 & ,(cn "0x3FF")) + ?\x40) << 10))
+		 (r6 &= ?\x3F)
+		 (r4 = ((r4 & ?\x03) << 6) | r6)
 		 (r0 |=  r4))))))
 
 (defun mucs-ccl-read-utf-16 ()
@@ -175,21 +186,22 @@
 ;UTF 16 Little Endian----------------------------------
 
 (defvar utf-16-le-ccl-encode
-  `((if (r0 < ?\xFFFF)
+  `((if (r0 < ,(cn "0xFFFF"))
 	,mucs-ccl-write-ex-le-2-octet
-      ((r4 = (((r0 >> 16) - 1) & ?\xF))
+      ((r4 = (((r0 >> 16) - 1) & ?\x0F))
        (r6 = ((r0 >> 10) & ?\x3F))
-       (write ((r4 & ?\x3) | r6))
-       (write ((r4 >> 2) | ?\xd8))
+       (write ((r4 & ?\x03) | r6))
+       (write ((r4 >> 2) | ?\xD8))
        (write (r0 & ?\xFF))
-       (write (((r0 >> 8) & ?\x3) | ?\xDC))))))
+       (write (((r0 >> 8) & ?\x03) | ?\xDC))))))
 
 (defvar utf-16-le-ccl-decode
   (append mucs-ccl-read-ex-le-2-octet
     `((if ,(utf-16-ccl-surrogate-pair-p 'r0)
 	  ((read r6 r4)
-	   (r4 = ((r4 & ?\x3) << 8) | r6)
-	   (r0 = ((((r0 & ?\x3FF) + ?\x40) << 10) | r4)))))))
+	   (r4 = ((r4 & ?\x03) << 8) | r6)
+	   (r0 = ((((r0 & ,(cn "0x3FF")) + ?\x40) << 10)
+		  | r4)))))))
 
 (mucs-type-register-serialization
  'ucs-generic
@@ -215,17 +227,18 @@
   (append mucs-ccl-read-ex-be-2-octet
     `((if ,(utf-16-ccl-surrogate-pair-p 'r0)
 	  ((read r4 r6)
-	   (r4 = ((r4 & ?\x3) << 8) | r6)
-	   (r0 = ((((r0 & ?\x3FF) + ?\x40) << 10) | r4)))))))
+	   (r4 = ((r4 & ?\x03) << 8) | r6)
+	   (r0 = ((((r0 & ,(cn "0x3FF")) + ?\x40) << 10)
+		  | r4)))))))
 
 (defvar utf-16-be-ccl-encode
-  `((if (r0 < ?\xffff)
+  `((if (r0 < ,(cn "0xFFFF"))
 	,mucs-ccl-write-ex-be-2-octet
-      ((r4 = (((r0 >> 16) - 1) & ?\xF))
+      ((r4 = (((r0 >> 16) - 1) & ?\x0F))
        (r6 = ((r0 >> 10) & ?\x3F))
-       (write ((r4 >> 2) | ?\xd8))
-       (write ((r4 & ?\x3) | r6))
-       (write (((r0 >> 8) & ?\x3) | ?\xDC))
+       (write ((r4 >> 2) | ?\xD8))
+       (write ((r4 & ?\x03) | r6))
+       (write (((r0 >> 8) & ?\x03) | ?\xDC))
        (write (r0 & ?\xFF))))))
 
 (mucs-type-register-serialization
@@ -245,7 +258,7 @@
  'none)
 
 (defun mucs-ccl-write-utf-16-be-signature ()
-  '((write ?\xfe) (write ?\xff)))
+  '((write "\xFE\xFF")))
 
 ;UTF 7 ------------------------------------------------
 
@@ -352,9 +365,11 @@
     (setq result
 	  (append
 	   result
-	   `((if (,valc > ?\xFFFF)
-		 ((r0 = (((,valc >> 10) - ?\x40) & ?\x3FF) | ?\xD800)
-		  (,valc = ((,valc & ?\x3FF) | ?\xDC00)))
+	   `((if (,valc > ,(cn "0xFFFF"))
+		 ((r0 = ((((,valc >> 10) - ?\x40)
+			  & ,(cn "0x3FF")) | ,(cn "0xD800")))
+		  (,valc = ((,valc & ,(cn "0x3FF"))
+			    | ,(cn "0xDC00"))))
 	       ((r0 = ,valc)
 		(,valc = 0)))
 	     (loop
@@ -363,18 +378,18 @@
 		   (write ,valp ,ccl-b64-encode-table)
 		   (,valp = ((r0 >> 4) & ?\x3F))
 		   (write ,valp ,ccl-b64-encode-table)
-		   (,valp = ((r0 & ?\xF) | ?\x30)))
+		   (,valp = ((r0 & ?\x0F) | ?\x30)))
 		((if (,valp >= ?\x30) ;; 4bit
-		     ((,valp = ((,valp & ?\xF) << 2))
+		     ((,valp = ((,valp & ?\x0F) << 2))
 		      (,valp |= (r0 >> 14))
 		      (write ,valp ,ccl-b64-encode-table)
 		      (,valp = ((r0 >> 8) & ?\x3F))
 		      (write ,valp ,ccl-b64-encode-table)
 		      (,valp = ((r0 >> 2) & ?\x3F))
 		      (write ,valp ,ccl-b64-encode-table)
-		      (,valp = ((r0 & ?\x3) | ?\x20)))
+		      (,valp = ((r0 & ?\x03) | ?\x20)))
 		   ((if (,valp >= ?\x20) ;; 2bit
-			((,valp = ((,valp & ?\x3) << 4))
+			((,valp = ((,valp & ?\x03) << 4))
 			 (,valp |= (r0 >> 12))
 			 (write ,valp ,ccl-b64-encode-table)
 			 (,valp = ((r0 >> 6) & ?\x3F))
@@ -436,7 +451,7 @@
 	       (r1 = r0 ,ccl-b64-decode-table)
 	       (if (r1 >= 0)
 		   ((r6 = ((r6 << 6) | r1))
-		    (if (r6 >= ?\x30000)
+		    (if (r6 >= ,(cn "0x30000"))
 			((r0 = (r6 & ?\xFFFF))
 			 (r6 = 0))
 		      ((read r0)
@@ -445,10 +460,10 @@
 			   ((r6 = ((r6 << 6) | r1))
 			    (if (r6 >= ?\x800000)
 				;; r6 = 1000 XXXX XXXX XXXX XXXX XXXX
-				((r0 = ((r6 >> 4) & ?\xFFFF))
+				((r0 = ((r6 >> 4) & ,(cn "0xFFFF")))
 				 (r6 = ((r6 & ?\xF) | ?\x30)))
 			      ;; r6 = XXXX XXXX XXXX XXXX XX
-			      ((r0 = ((r6 >> 2) & ?\xFFFF))
+			      ((r0 = ((r6 >> 2) & ,(cn "0xFFFF")))
 			       (r6 = ((r6 & ?\x3) | ?\x20)))))
 			 ,(mucs-ccl-set-internal-state
 			   'utf-7-shifted-p nil)))))
@@ -472,15 +487,16 @@
 				   (if (r1 >= 0)
 				       ((r6 = ((r6 << 6) | r1))
 					(r0 = (r6 >> 2))
-					(r6 = ((r6 & ?\x3) | ?\x20))
+					(r6 = ((r6 & ?\x03) | ?\x20))
 					,@(mucs-ccl-set-internal-state
 					   'utf-7-shifted-p t)))))))))))))
      (if ,(utf-16-ccl-surrogate-pair-p 'r0)
 	 ((r4 = r0)
 	  (repeat)))
      (if (r4 != 0)
-	 ((r4 &= ?\x3FF)
-	  (r0 = (((r0 & ?\x3FF) + ?\x40) << 10) | r4))))))
+	 ((r4 &= ,(cn "0x3FF"))
+	  (r0 = ((((r0 & ,(cn "0x3FF")) + ?\x40) << 10)
+		 | r4)))))))
 
 (mucs-type-register-serialization
  'ucs-generic
